<?php
namespace Teknisa\CRM\Service;

use Zeedhi\Framework\HTTP\Kernel;
use Doctrine\ORM\EntityManager;
use Teknisa\CRM\Factory\ImportacaoProspectFactory;

class ImportacaoProspectService
{
    protected $entityManager;
    /** @var Environment */
    protected $environment;
    protected $tempDirectoryPath;
    protected $importacaoProspectFactory;
    
    const TIPODEPARCEIRO_LEAD = "CANDIDATO";
    
    public function __construct(Kernel $httpKernel, EntityManager $entityManager, $environment, $tempDirectoryPath, ImportacaoProspectFactory $importacaoProspectFactory) {
        $this->httpKernel = $httpKernel;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->tempDirectoryPath = $tempDirectoryPath;
        $this->importacaoProspectFactory = $importacaoProspectFactory;
    }
    
    /**
     * Realiza o upload do arquivo selecionado e inicia a importação dos dados
     *
     * @param $dataset
     * @throws \Exception
     */
    public function saveUploadFile($dataset) {
        try {
            $arquivos = $dataset['FILEPATH'];
            unset($dataset['FILEPATH']);
            
            foreach ($arquivos as $fileData) {
                // realiza upload do arquivo no diretório padrão de uploads
                $file = $this->httpKernel->factoryFromFileData($fileData, $this->tempDirectoryPath);
            }        
    
            if ($file->isFile()) {
                return $this->iniciarImportacao($file->getPathname(), (array)$dataset);
            } else {
                throw new \Exception("Falha ao gravar o arquivo.");
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     * Inicia a rotina de importação dos registros
     *
     * @param $filePath
     * @throws \Exception
     */
    private function iniciarImportacao($filePath, $dataset) {
        try {
            $nrorgtrab = $this->environment->getNrOrgTrab();
            $prospectMapping = realpath(__DIR__."/../ExcelMapping/prospectMapping.json");
            
            // verifica se o arquivo existe
            if (!file_exists($filePath)) {
                throw new \Exception("Arquivo não encontrado");
            }
            
            // retorna os registros do arquivo
            $fileData = $this->getDataFile($filePath);
            
            // valida se as colunas do excel existem no agrupamento do estrutura.json
            $this->validateColumns($fileData, $prospectMapping, 'PROSPECT');
            
            // grava os dados
            $errors = array();
            $success = array();
            $count = 0;
            foreach ($fileData as $data) {
                
                foreach ($data as $key => $value) {
                    if (empty($key))
                        unset($data[$key]);
                }
                
                $connection = $this->entityManager->getConnection();
                
                try {
                    $connection->beginTransaction();
                    
                    $this->importacaoProspectFactory->setDataset(array_merge($data, $dataset));
                    $this->importacaoProspectFactory->getOperadorResponsavel();
                    $this->importacaoProspectFactory->setLead();
                    if($dataset['TIPODEPARCEIRO'] != self::TIPODEPARCEIRO_LEAD){
                        $this->importacaoProspectFactory->setProspect();
                        $this->importacaoProspectFactory->setEvolucao();
                        $this->importacaoProspectFactory->setTarefa();
                    }
                    
                    //guarda os prospects devidamente importados
                    $success[$count]['NRINSJURPROS']     = $data['CNPJ'];
                    $success[$count]['NMRAZSOCPROS']     = $data['RAZAO_SOCIAL'];
                    
                    $connection->commit();
                } catch (\Exception $e) {
                    $connection->rollBack();
                    
                    // guarda o erro pelo cnpj do prospect
                    $errors[$count]['NRINSJURPROS']     = $data['CNPJ'];
                    $errors[$count]['NMRAZSOCPROS']     = $data['RAZAO_SOCIAL'];
                    $errors[$count]['DSINCONSISTENCIA'] = $e->getMessage();
                }
                
                $count++;
            }
        
            if (empty($errors))
                return true;
            else
                return array("error" => $errors, "success" => $success);
                
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     * Retorna todos os registros do arquivo selecionado
     *
     * @param $filePath
     */
    public function getDataFile($filePath) {
        $extensionFile = $this->getExtensionFile($filePath);
        
        if ($extensionFile == 'xls' || $extensionFile == 'xlsx') {

            $xls  = new \Teknisa\CRM\Util\ExcelUtils($filePath);
            $data = $xls->buildDataIntoJson();

        } elseif ($extensionFile == 'csv') {

            $delimitador = "|";
            $cerca       = '"';
            $csv         = new \Importacao\Logic\Util\CSVUtils($filePath, $delimitador, $cerca);
            $data        = $csv->buildDataIntoJson();

        } else {
            throw new \Exception("Tipo de arquivo ". $extensionFile ." não suportado.");
        }
        
        return $data;
    }
    
    /**
     * Retorna a extensão do arquivo selecionado
     *
     * @param $file
     */
    private function getExtensionFile($file) {
        $file = explode('.', $file);
        return end($file);
    }
    
    /**
     * Valida se o nome das colunas no excel existem no agrupamento do prospectMapping.json
     *
     * @param $data
     * @param $estruturaJson
     * @param $agrupamento
     * @throws \Exception
     */
    private function validateColumns($data, $jsonMapping, $agrupamento) {
        $jsonMapping = json_decode(file_get_contents($jsonMapping), true);
        
        $colunasJson = array();
        foreach ($jsonMapping as $nomeAgrupamento => $colunas) {
            if ($agrupamento === $nomeAgrupamento) {
                $colunasJson = array_keys($colunas);
            }
        }
        
        $count = 0;
        foreach ($data as $key => $linha) {
            foreach ($linha as $nomeColunaExcel => $valorColunaExcel) {
                $linha[$nomeColunaExcel] =
                    $this->removeEmptyCharacters($valorColunaExcel);
                if (!empty($nomeColunaExcel)) {
                    foreach ($colunasJson as $nomeColunaJson) {
                        if (strpos($nomeColunaJson, $nomeColunaExcel) || $nomeColunaJson == $nomeColunaExcel) {
                            $count++;
                        }
                    }
    
                    if ($count == 0) {
                        throw new \Exception("A coluna '$nomeColunaExcel' não existe.");
                    }
    
                    $count = 0;
                }
            }
            break;
        }
    }
    
    /**
     * Remove os espaços repetidos ou contidos no começo ou fim da string.
     * Ex: ' Hoje  está um  belo dia  ' --> 'Hoje está um belo dia'.
     * 
     * @param string $oriValue - String a ser tratada.
     * 
     * @returns string - String tratada.
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    public function removeEmptyCharacters($oriValue) {
        $formatedValue = '';
        
        for ($index = 0; $index < strlen($oriValue); $index++) {
            $char = $oriValue[$index];
            
            $previousChar = strlen($formatedValue) == 0 ?
                ' ' : $formatedValue[sizeof($formatedValue) - 1];
            
            $nextChar = $index == strlen($oriValue) - 1 ?
                ' ' : $oriValue[$index + 1];
            
            if($char != ' ' || $previousChar != ' ' && $nextChar != ' ') {
                $formatedValue .= $char;
            }
        }
        
        return $formatedValue;
    }
}